/*
 *    HardInfo - Displays System Information
 *    Copyright (C) 2003-2008 Leandro A. F. Pereira <leandro@hardinfo.org>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 2.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include <config.h>
#include <time.h>
#include <string.h>
#include <sys/utsname.h>
#include <sys/stat.h>
#include "po.h"
#include <math.h>


#include <hardinfo.h>
#include <iconcache.h>
#include <shell.h>

#include <vendor.h>

/* Callbacks */
gchar *callback_summary();
gchar *callback_os();
gchar *callback_modules();
/*NeoKylin add by zhang yujie*/
gchar *callback_uptimed();
/*NeoKylin done by zhang yujie*/
gchar *callback_boots();
gchar *callback_locales();
gchar *callback_fs();
gchar *callback_display();
gchar *callback_network();
gchar *callback_users();
gchar *callback_env_var();

/* Scan callbacks */
void scan_summary(gboolean reload);
void scan_os(gboolean reload);
void scan_modules(gboolean reload);
void scan_boots(gboolean reload);
/*NeoKylin add by zhang yujie*/
void scan_uptimed(gboolean reload);
/*NeoKylin add by zhang yujie*/
void scan_locales(gboolean reload);
void scan_fs(gboolean reload);
void scan_display(gboolean reload);
void scan_network(gboolean reload);
void scan_users(gboolean reload);
void scan_env_var(gboolean reload);

static ModuleEntry entries[] = {
    {"Summary", "summary.png", callback_summary, scan_summary},
    {"Operating System", "os.png", callback_os, scan_os},
    {"Kernel Modules", "module.png", callback_modules, scan_modules},
    {"Boots", "boot.png", callback_boots, scan_boots},
    {"Uptimed", "uptimed.png", callback_uptimed, scan_uptimed},
    /*{"Languages", "language.png", callback_locales, scan_locales},*/
    {"Filesystems", "dev_removable.png", callback_fs, scan_fs},
    {"Display", "monitor.png", callback_display, scan_display},
    {"Environment Variables", "environment.png", callback_env_var, scan_env_var},
    {"Users", "users.png", callback_users, scan_users},
    {NULL},
};

#include "computer.h"

static GHashTable *moreinfo = NULL;
static gchar *module_list = NULL;
static Computer *computer = NULL;

#include <arch/this/modules.h>
#include <arch/common/languages.h>
#include <arch/this/alsa.h>
#include <arch/common/display.h>
#include <arch/this/loadavg.h>
#include <arch/this/memory.h>
#include <arch/this/uptime.h>
#include <arch/this/os.h>
#include <arch/this/filesystem.h>
#include <arch/common/users.h>
#include <arch/this/boots.h>
#include <arch/common/environment.h>

/*NeoKylin add by zhang yujie*/
gchar e_mon[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
gchar c_mon[12][3]={"01","02","03","04","05","06","07","08","09","10","11","12"};
gchar e_week[7][4]={"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
gchar c_week[7][10]={"一","二","三","四","五","六","日"};
gchar c_data[100];

gchar *mon_to_ch(gchar *month)
 {
        char *p_mon=month;
        int i = 0;
        for(i = 0;i < 12;i++){
                if((strncmp(p_mon,e_mon[i],3)) == 0)
                        return c_mon[i];
        }
	return NULL;
}
gchar *week_to_ch(gchar *week)
 {
        char *p_week=week;
        int i = 0;
        for(i = 0;i < 7;i++){
                if((strncmp(p_week,e_week[i],3)) == 0)
                        return c_week[i];
        }
	return NULL;
}
gchar *data_to_ch(gchar *time,gchar *year,gchar *week,gchar *month,gchar *day)
{
        memset(c_data,0,sizeof(c_data));
        sprintf(c_data,"%s年 %s月 %02d日 星期%s  %s",year,mon_to_ch(month),atoi(day),week_to_ch(week),time);

        return c_data;
}
/*NeoKylin done by zhang yujie*/


gchar *hi_more_info(gchar * entry)
{
    gchar *info = (gchar *) g_hash_table_lookup(moreinfo, entry);

    if (info)
	return g_strdup(info);

    return g_strdup_printf("[%s]", entry);
}

gchar *hi_get_field(gchar * field)
{
    gchar *tmp;
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
    {
    if (g_str_equal(field, "内存")) {
       MemoryInfo *mi = computer_get_memory();
       double memt = mi->total >> 10;
       tmp = g_strdup_printf("%.2fGB", ceil(memt));
       g_free(mi);
    } else if (g_str_equal(field, "运行时间")) {
       tmp = computer_get_formatted_uptime();
    } else if (g_str_equal(field, "系统时间")) {
       time_t t = time(NULL);

       tmp = g_new0(gchar, 64);
       strftime(tmp, 64, "%c", localtime(&t));
    } else if (g_str_equal(field, "负载均衡")) {
       tmp = computer_get_formatted_loadavg();
    } else {
       tmp = g_strdup("");
    }
       }
    else
    {

    if (g_str_equal(field, "Memory")) {
	MemoryInfo *mi = computer_get_memory();
        double memt = mi->total*0.000976562; 
	tmp = g_strdup_printf("%.2fGB (%.2fGB useable)", ceil(memt), memt);
	g_free(mi);
    } else if (g_str_equal(field, "Uptime")) {
	tmp = computer_get_formatted_uptime();
    } else if (g_str_equal(field, "Date/Time")) {
	time_t t = time(NULL);

	tmp = g_new0(gchar, 64);
	strftime(tmp, 64, "%c", localtime(&t));
    } else if (g_str_equal(field, "Load Average")) {
	tmp = computer_get_formatted_loadavg();
    } else {
	tmp = g_strdup("");
    }
  }
    return tmp;
}

void scan_summary(gboolean reload)
{
    SCAN_START();
    module_entry_scan_all_except(entries, 0);
    computer->alsa = computer_get_alsainfo();
    SCAN_END();
}

void scan_os(gboolean reload)
{
    SCAN_START();
    computer->os = computer_get_os();
    SCAN_END();
}

void scan_modules(gboolean reload)
{
    SCAN_START();
    scan_modules_do();
    SCAN_END();
}

void scan_boots(gboolean reload)
{
    SCAN_START();
    scan_boots_real();
    SCAN_END();
}

void __delete_blank(gchar *str)
{
    gchar *sp = str;
    gchar *prev = 0;

    while(*str && (*str == ' ' || *str == '\n' || *str =='\t'))
        ++str;

    for(; *str; ++str)
    {
        switch (*str)
        {
        case '\t':  *str = ' ';
        case ' ' :
            if (*prev == '\n' || *prev ==' ')
                continue;
            break;

        case '\n':
            if (*prev == '\n')
                continue;
            else if (*prev == ' ')
            {
                *prev = '\n';
                continue;
            }
            break;
        }

        prev = sp;
        *sp++ = *str;
    }

    if(prev && *prev && (*prev == ' ' || *prev == '\n'))
        --sp;
    *sp = 0;

}

/*NeoKylin add by zhang yujie*/
void scan_uptimed(gboolean reload)
{
//    FILE *uprecords_fd;
    FILE *sed_fd;
    gchar buffer[128];
    gchar cmd_buf[100];
    gchar ch_flg = 0;
    scan_os(FALSE);
    sprintf(cmd_buf,"sed -r %c",'"');
    strcat(cmd_buf,"s/\\x1B\\[([0-9]{1,2}(;[0-9]{1,2})?)?[m|K]//g");
    char k = strlen(cmd_buf);
    *(cmd_buf+k)='"';
    *(cmd_buf+k+1)='\0';
    strcat(cmd_buf," /usr/share/hardinfo_uptimed");
//    printf("%s\n",cmd_buf);
//	if (computer->os->uptimed != NULL)
  //   		 return;

    memset(buffer,0,sizeof(buffer));
	  SCAN_START();
	 g_free(computer->os->uptimed);
   	computer->os->uptimed = g_strdup("");
	 g_free(computer->os->uptimed_count);
   	computer->os->uptimed_count = g_strdup("");

    if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
	ch_flg = 1;
    else
	ch_flg = 0;
	system("uprecords > /usr/share/hardinfo_uptimed && sync");
//    uprecords_fd = popen("uprecords > /usr/share/hardinfo_uptimed","r");
//    if(uprecords_fd){
	sed_fd = popen(cmd_buf,"r");
	if(sed_fd){
	    while (fgets(buffer+2, 128, sed_fd)) {
		*buffer = 'l';
		*(buffer+1) = ' ';
		gchar **tmp = NULL,*buf = buffer;
		strend(buffer,'\n');
		while (*buf) {
		    if ((*buf == '|')||(*buf == '+')) {
		//	strcpy(buf, buf + 1);
			*buf=' ';
			buf++;
		    }else {
			if(*buf == ',')
			    *buf=' ';
			buf++;
		    }
		}
		printf("%s\n",buffer);
		buf=buffer;
/*
		while (*buf) {
		    if (*buf == ' ' && *(buf + 1) == ' ') {
		        strcpy(buf, buf + 1);
		//        buf--;
	            } else {
		        buf++;
		    }
		}
*/
		__delete_blank(buf);

		printf("%s\n",buffer);

	        if(strstr(buffer,"Linux")){
		    tmp = g_strsplit(buffer," ",0);
		    if((*(tmp[1]) == '-') && ((*(tmp[1]+1)) == '>')){
			if(ch_flg == 1){
			    computer->os->uptimed = h_strdup_cprintf("%s      %s=%s|%3d  天  %s    |%s  %s    \n",
			    computer->os->uptimed,tmp[2],tmp[1],data_to_ch(tmp[11],tmp[12],tmp[8],tmp[9],tmp[10]),atoi(tmp[3]),tmp[5],tmp[6],tmp[7]);
			}else{
			    computer->os->uptimed = h_strdup_cprintf("%s      %s=%s  %s  %s  %s  %s|%s  %s  %s    |%s  %s    \n",
			    computer->os->uptimed,tmp[2],tmp[1],tmp[11],tmp[12],tmp[8],tmp[9],tmp[10],tmp[3],tmp[4],tmp[5],tmp[6],tmp[7]);
		        }
		    }else{
			if(ch_flg == 1){
			    computer->os->uptimed = h_strdup_cprintf("%s=%s|%3d  天  %s    |%s  %s    \n",
			    computer->os->uptimed,tmp[1],data_to_ch(tmp[10],tmp[11],tmp[7],tmp[8],tmp[9]),atoi(tmp[2]),tmp[4],tmp[5],tmp[6]);
		        }else{
			    computer->os->uptimed = h_strdup_cprintf("%s=%s  %s  %s  %s  %s|%s  %s  %s    |%s  %s    \n",
			    computer->os->uptimed,tmp[1],tmp[10],tmp[11],tmp[7],tmp[8],tmp[9],tmp[2],tmp[3],tmp[4],tmp[5],tmp[6]);
			}
		    }
		} else if(strstr(buffer,"at")){
		    tmp = g_strsplit(buffer," ",0);
		    if((*(tmp[1]) == '1') && ((*(tmp[1]+1)) == 'u')){
		        if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("与前者差距=%s|%3d  天  %s    |目标时间点  \n",
			    computer->os->uptimed_count,data_to_ch(tmp[10],tmp[11],tmp[7],tmp[8],tmp[9]),atoi(tmp[3]),tmp[5]);
		        }else{
			    computer->os->uptimed_count = h_strdup_cprintf("1up  in=%s  %s  %s  %s  %s|%s  %s  %s    |%s  \n",
			    computer->os->uptimed_count,tmp[10],tmp[11],tmp[7],tmp[8],tmp[9],tmp[3],tmp[4],tmp[5],tmp[6]);
		        }
		    }else if((*(tmp[1]) == 'n') && ((*(tmp[1]+1)) == 'o')){
			if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("与第一差距=%s|%3d  天  %s    |目标时间点  \n",
			    computer->os->uptimed_count,data_to_ch(tmp[10],tmp[11],tmp[7],tmp[8],tmp[9]),atoi(tmp[3]),tmp[5]);
			}else{
			    computer->os->uptimed_count = h_strdup_cprintf("no1  in=%s  %s  %s  %s  %s|%s  %s  %s    |%s  \n",
			    computer->os->uptimed_count,tmp[10],tmp[11],tmp[7],tmp[8],tmp[9],tmp[3],tmp[4],tmp[5],tmp[6]);
			}
		    }else if((*(tmp[1]) == 't') && ((*(tmp[1]+1)) == '1')){
			if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("与第十差距=%s|%3d  天  %s    |目标时间点  \n",
			    computer->os->uptimed_count,data_to_ch(tmp[10],tmp[11],tmp[7],tmp[8],tmp[9]),atoi(tmp[3]),tmp[5]);
			}else{
			    computer->os->uptimed_count = h_strdup_cprintf("t10 in=%s  %s  %s  %s  %s|%s  %s  %s    |%s  \n",
			    computer->os->uptimed_count,tmp[10],tmp[11],tmp[7],tmp[8],tmp[9],tmp[3],tmp[4],tmp[5],tmp[6]);
			}
		    }
		} else if(strstr(buffer,"since")){
		    tmp = g_strsplit(buffer," ",0);
		    if((*(tmp[1]) == '%') && ((*(tmp[1]+1)) == 'u')){
			if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("运行时间比=%s|%s%%    |统计起始于  \n",
			    computer->os->uptimed_count,data_to_ch(tmp[7],tmp[8],tmp[4],tmp[5],tmp[6]),tmp[2]);
			}else{
			    computer->os->uptimed_count = h_strdup_cprintf("%%up=%s  %s  %s  %s  %s|%s%%    |%s    \n",
			    computer->os->uptimed_count,tmp[7],tmp[8],tmp[4],tmp[5],tmp[6],tmp[2],tmp[3]);
		        }
		    }else if((*(tmp[1]) == 'u') && ((*(tmp[1]+1)) == 'p')){
			if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("运行总时间=%s|%3d  天  %s    |统计起始于\n",
			    computer->os->uptimed_count,data_to_ch(tmp[9],tmp[10],tmp[6],tmp[7],tmp[8]),atoi(tmp[2]),tmp[4]);
			}else{
			    computer->os->uptimed_count = h_strdup_cprintf("up=%s  %s  %s  %s  %s|%s  %s  %s    |%s    \n",
			    computer->os->uptimed_count,tmp[9],tmp[10],tmp[6],tmp[7],tmp[8],tmp[2],tmp[3],tmp[4],tmp[5]);
		        }
		    }else if((*(tmp[1]) == 'd') && ((*(tmp[1]+1)) == 'o')){
			if(ch_flg == 1){
			    computer->os->uptimed_count = h_strdup_cprintf("停机总时间=%s|%3d  天  %s    |统计起始于  \n",
			    computer->os->uptimed_count,data_to_ch(tmp[9],tmp[10],tmp[6],tmp[7],tmp[8]),atoi(tmp[2]),tmp[4]);
			}else{
			    computer->os->uptimed_count = h_strdup_cprintf("down=%s  %s  %s  %s  %s|%s  %s  %s    |%s    \n",
			    computer->os->uptimed_count,tmp[9],tmp[10],tmp[6],tmp[7],tmp[8],tmp[2],tmp[3],tmp[4],tmp[5]);
			}
		    }
		}
	    }
//		    printf("@@@@@@@@@@@@@@@@@@@@@@@@@@\n%s\n",computer->os->uptimed);
	}
//    }
    pclose(sed_fd);
 //   pclose(uprecords_fd);
    SCAN_END();

}
/*NeoKylin done by zhang yujie*/



void scan_locales(gboolean reload)
{
	SCAN_START();
	scan_os(FALSE);
	scan_languages(computer->os);
	SCAN_END();
}

void scan_fs(gboolean reload)
{
	SCAN_START();
	scan_filesystems();
	SCAN_END();
}

void scan_display(gboolean reload)
{
	SCAN_START();
	computer->display = computer_get_display();
	SCAN_END();
}

void scan_users(gboolean reload)
{
	SCAN_START();
	scan_users_do();
	SCAN_END();
}

gchar *callback_summary()
{
	gchar *processor_name, *alsa_cards, *input_devices, *printers, *storage_devices, *summary;

	processor_name  = module_call_method("devices::getProcessorName");
	alsa_cards      = computer_get_alsacards(computer);
	input_devices   = module_call_method("devices::getInputDevices");
	printers        = module_call_method("devices::getPrinters");
	storage_devices = module_call_method("devices::getStorageDevices");
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
		summary = g_strdup_printf("[$ShellParam$]\n"
				"UpdateInterval$Memory=1000\n"
				"UpdateInterval$Date/Time=1000\n"
				"#ReloadInterval=5000\n"
				"[计算机]\n"
				"处理器=%s\n"
				"内存=...\n"
				"操作系统=%s\n"
				"用户信息=%s\n"
				"系统时间=...\n"
				"[显示]\n"
				"分辨率=%dx%d pixels\n"
				"图形控制=%s\n"
				"[多媒体]\n"
				"\n%s\n"
				"[输入设备]\n%s\n"
				"\n%s\n"
				"\n%s\n",
				processor_name,
				computer->os->distro,
				computer->os->username,
				computer->display->width,
				computer->display->height,
				computer->display->vendor,
				alsa_cards,
				input_devices, printers, storage_devices);
	else
		summary = g_strdup_printf("[$ShellParam$]\n"
				"UpdateInterval$Memory=1000\n"
				"UpdateInterval$Date/Time=1000\n"
				"#ReloadInterval=5000\n"
				"[Computer]\n"
				"Processor=%s\n"
				"Memory=...\n"
				"Operating System=%s\n"
				"User Name=%s\n"
				"Date/Time=...\n"
				"[Display]\n"
				"Resolution=%dx%d pixels\n"
				"X11 Vendor=%s\n"
				"[Multimedia]\n"
				"\n%s\n"
				"[Input Devices]\n%s\n"
				"\n%s\n"
				"\n%s\n",
				processor_name,
				computer->os->distro,
				computer->os->username,
				computer->display->width,
				computer->display->height,
				computer->display->vendor,
				alsa_cards,
				input_devices, printers, storage_devices);

	g_free(processor_name);
	g_free(alsa_cards);
	g_free(input_devices);
	g_free(printers);
	g_free(storage_devices);

	return summary;
}

gchar *callback_os()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
		return g_strdup_printf("[$ShellParam$]\n"
				"UpdateInterval$Uptime=10000\n"
				"UpdateInterval$Load Average=1000\n"
				"[版本信息]\n"
				"内核版本=%s\n"
				"编译日期=%s\n"
				"C库版本=%s\n"
				"C编译器=%s\n"
				"系统描述=%s\n"
				"[环境信息]\n"
				"计算机名=%s\n"
				"用户信息=%s\n"
				"#Language=%s\n"
				"用户目录=%s\n"
				"桌面环境=%s\n"
				"[运行信息]\n"
				"运行时间=...\n"
				"负载均衡=...",
				computer->os->kernel,
				computer->os->compiled_date,
				computer->os->libc,
				computer->os->gcc,
				computer->os->distro,
				computer->os->hostname,
				computer->os->username,
				computer->os->language,
				computer->os->homedir, computer->os->desktop);
	else
		return g_strdup_printf("[$ShellParam$]\n"
				"UpdateInterval$Uptime=10000\n"
				"UpdateInterval$Load Average=1000\n"
				"[Version]\n"
				"Kernel=%s\n"
				"Compiled=%s\n"
				"C Library=%s\n"
				"Default C Compiler=%s\n"
				"Distribution=%s\n"
				"[Current Session]\n"
				"Computer Name=%s\n"
				"User Name=%s\n"
				"#Language=%s\n"
				"Home Directory=%s\n"
				"Desktop Environment=%s\n"
				"[Misc]\n"
				"Uptime=...\n"
				"Load Average=...",
				computer->os->kernel,
				computer->os->compiled_date,
				computer->os->libc,
				computer->os->gcc,
				computer->os->distro,
				computer->os->hostname,
				computer->os->username,
				computer->os->language,
				computer->os->homedir, computer->os->desktop);
}


/*NeoKylin add by zhang yujie*/
gchar *callback_modules()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8"))){
		return g_strdup_printf("[内核模块]\n"
				"%s"
				"[$ShellParam$]\n"
				"ViewType=1\n"
				"ColumnTitle$TextValue=名称\n"
				"ColumnTitle$Value=描述\n"
				"ShowColumnHeaders=true\n", module_list);
	}else{
		return g_strdup_printf("[Loaded Modules]\n"
				"%s"
				"[$ShellParam$]\n"
				"ViewType=1\n"
				"ColumnTitle$TextValue=Name\n"
				"ColumnTitle$Value=Description\n"
				"ShowColumnHeaders=true\n", module_list);
	}
}

gchar *callback_boots()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8"))){
		return g_strdup_printf("[$ShellParam$]\n"
				"ColumnTitle$TextValue=日期和时间\n"
				"ColumnTitle$Value=内核版本\n"
				"ShowColumnHeaders=true\n"
				"[启动信息]\n"
				"%s", computer->os->boots);
	}else{
		return g_strdup_printf("[$ShellParam$]\n"
				"ColumnTitle$TextValue=Date & Time\n"
				"ColumnTitle$Value=Kernel Version\n"
				"ShowColumnHeaders=true\n"
				"[Boots]\n"
				"%s", computer->os->boots);
	}
}

gchar *callback_uptimed()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8"))){
		return g_strdup_printf("[$ShellParam$]\n"
				"ColumnTitle$TextValue=序号\n"
				"ColumnTitle$Value=开机时间\n"
				"ColumnTitle$Extra1=运行时间\n"
				"ColumnTitle$Extra2=内核版本\n"
				"ShowColumnHeaders=true\n"
				"[运行时间]\n"
				"%s"
				"---=---|---|---\n"
				"%s",
				computer->os->uptimed,
				computer->os->uptimed_count
				);

	}
	else{
		return g_strdup_printf("[$ShellParam$]\n"
				"ColumnTitle$TextValue=ID\n"
				"ColumnTitle$Value=Boot up\n"
				"ColumnTitle$Extra1=Uptime\n"
				"ColumnTitle$Extra2=System\n"
				"ShowColumnHeaders=true\n"
				"[Uptimed]\n"
				"%s"
				"---=---|---|---\n"
				"%s\n",
				computer->os->uptimed,
				computer->os->uptimed_count
				);
	}
}


gchar *callback_locales()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8"))){

		return g_strdup_printf("[$ShellParam$]\n"
				"ViewType=1\n"
				"ColumnTitle$TextValue=语言编码\n"
				"ColumnTitle$Value=名称\n"
				"ShowColumnHeaders=true\n"
				"[语言]\n"
				"%s", computer->os->languages);
	}else{
		return g_strdup_printf("[$ShellParam$]\n"
				"ViewType=1\n"
				"ColumnTitle$TextValue=Language Code\n"
				"ColumnTitle$Value=Name\n"
				"ShowColumnHeaders=true\n"
				"[Available Languages]\n"
				"%s", computer->os->languages);
	}
}

gchar *callback_fs()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8"))){
		return g_strdup_printf("[$ShellParam$]\n"
				"ViewType=4\n"
				"ReloadInterval=5000\n"
				"Zebra=1\n"
				"NormalizePercentage=false\n"
				"ColumnTitle$Extra1=挂载点\n"
				"ColumnTitle$Progress=使用情况\n"
				"ColumnTitle$TextValue=设备\n"
				"ShowColumnHeaders=true\n"
				"[文件系统]\n%s\n", fs_list);
	}else{
		return g_strdup_printf("[$ShellParam$]\n"
				"ViewType=4\n"
				"ReloadInterval=5000\n"
				"Zebra=1\n"
				"NormalizePercentage=false\n"
				"ColumnTitle$Extra1=Mount Point\n"
				"ColumnTitle$Progress=Usage\n"
				"ColumnTitle$TextValue=Device\n"
				"ShowColumnHeaders=true\n"
				"[Mounted File Systems]\n%s\n", fs_list);
	}
}
/*NeoKylin done by zhang yujie*/

gchar *callback_display()
{
	if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
		return g_strdup_printf("[显示信息]\n"
				"分辨率=%dx%d pixels\n"
				"提供商=%s\n"
				"[屏幕信息]\n"
				"%s"
				"[扩展信息]\n"
				"%s"
				"[图形库]\n"
				"提供商=%s\n"
				"渲染工具=%s\n"
				"版本信息=%s\n"
				"直接渲染=%s\n",
				computer->display->width,
				computer->display->height,
				computer->display->vendor,
				computer->display->monitors,
				computer->display->extensions,
				computer->display->ogl_vendor,
				computer->display->ogl_renderer,
				computer->display->ogl_version,
				computer->display->dri ? "Yes" : "No");
	else
		return g_strdup_printf("[Display]\n"
				"Resolution=%dx%d pixels\n"
				"Vendor=%s\n"
				"[Monitors]\n"
				"%s"
				"[Extensions]\n"
				"%s"
				"[OpenGL]\n"
				"Vendor=%s\n"
				"Renderer=%s\n"
				"Version=%s\n"
				"Direct Rendering=%s\n",
				computer->display->width,
				computer->display->height,
				computer->display->vendor,
				computer->display->monitors,
				computer->display->extensions,
				computer->display->ogl_vendor,
				computer->display->ogl_renderer,
				computer->display->ogl_version,
				computer->display->dri ? "Yes" : "No");
}

gchar *callback_users()
{
	return g_strdup_printf("[$ShellParam$]\n"
			"ReloadInterval=10000\n"
			"ViewType=1\n"
			"[Users]\n"
			"%s\n", users);
}

gchar *get_os_kernel(void)
{
	scan_os(FALSE);
	return computer->os->kernel;
}

gchar *get_kernel_module_description(gchar *module)
{
	gchar *description;

	if (!_module_hash_table) {
		scan_modules(FALSE);
	}

	description = g_hash_table_lookup(_module_hash_table, module);
	if (!description) {
		return g_strdup(module);
	}

	return g_strdup(description);
}

ShellModuleMethod *hi_exported_methods(void)
{
	static ShellModuleMethod m[] = {
		{"getOSKernel", get_os_kernel},
		{"getKernelModuleDescription", get_kernel_module_description},
		{NULL}
	};

	return m;
}
/*NeoKylin add by zhang yujie*/
ModuleEntry *hi_module_get_entries(void)
{
	entries[0].name=_("Summary");
	entries[1].name=_("Operating System");
	entries[2].name=_("Kernel Modules");
	entries[3].name=_("Boots");
	entries[4].name=_("Uptimed");
	/*entries[5].name=_("Languages");*/
	entries[5].name=_("Filesystems");
	entries[6].name=_("Display");
	entries[7].name=_("Environment Variables");
	entries[8].name=_("Users");
	return entries;
}
/*NeoKylin done by zhang yujie*/

gchar *hi_module_get_name(void)
{
	return g_strdup(_("Computer"));
}

guchar hi_module_get_weight(void)
{
	return 80;
}

gchar **hi_module_get_dependencies(void)
{
	static gchar *deps[] = { "devices.so", NULL };

	return deps;
}

void hi_module_init(void)
{
	computer = g_new0(Computer, 1);
	moreinfo =
		g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
}

ModuleAbout *hi_module_get_about(void)
{
	static ModuleAbout ma[] = {
		{
			.author = "Leandro A. F. Pereira",
			.description = "Gathers high-level computer information",
			.version = VERSION,
			.license = "GNU GPL version 2"}
	};
	ma[0].description = _("Gathers high-level computer information");
	return ma;
}
